package com.engine.organization.service.impl;

import cn.hutool.core.convert.Convert;
import com.engine.core.impl.Service;
import com.engine.organization.entity.jingyou.*;
import com.engine.organization.service.OrgChartService;
import com.engine.organization.util.HasRightUtil;
import org.apache.commons.lang3.StringUtils;
import weaver.conn.RecordSet;
import weaver.hrm.User;

import java.util.*;

/**
 * @className: OrgChartServiceImpl
 * @author: dengjp
 * @date: 2022/7/7
 * @description: 组织架构图ServiceImpl
 **/
public class JyChartServiceImpl extends Service implements OrgChartService {

    private static final String COMPANY_RIGHT = "OrgChart:All";
    private static final String USER_RIGHT = "OrgPerspective:All";
    RecordSet rs = new RecordSet();

    @Override
    public Map<String, Object> getOptionCondition(Map<String, Object> request2Map, User user) {
        Map<String, Object> result = new HashMap<>();

        rs.executeQuery("select id, companyname from HrmCompanyVirtual order by id");
        List<Map<String, Object>> fclasslist = new ArrayList<>();
        Map<String, Object> defaultItem = new HashMap<>();
        defaultItem.put("id", "0");
        defaultItem.put("companyname", "行政维度");
        fclasslist.add(defaultItem);
        while (rs.next()) {
            Map<String, Object> item = new HashMap<>();
            item.put("id", rs.getString("id"));
            item.put("companyname", rs.getString("companyname"));
            fclasslist.add(item);
        }
        // 集团数据
        String sql = "select concat('c_',id) as id, companyname as fname  from hrmcompany";
        rs.executeQuery(sql);
        Set<JyOrgSelectItem> companySet = new HashSet<>();
        while (rs.next()) {
            JyOrgSelectItem item = new JyOrgSelectItem();
            item.setId("0");
            //item.setFnumber(rs.getString("fnumber"));
            item.setFname(rs.getString("fname"));
            companySet.add(item);
        }


        // 分部数据
        sql = "select concat('s_',id) as id,subcompanyname as fname, subcompanycode as fnumber from hrmsubcompany where ifnull(canceled,'0')='0' order by showorder ,showOrderOfTree";
        rs.executeQuery(sql);
        while (rs.next()) {
            JyOrgSelectItem item = new JyOrgSelectItem();
            item.setId(rs.getString("id"));
            item.setFnumber(rs.getString("fnumber"));
            item.setFname(rs.getString("fname"));
            companySet.add(item);
        }
        result.put("api_status", true);
        result.put("fclasslist", fclasslist);
        result.put("companylist", companySet);
        return result;
    }

    @Override
    public Map<String, Object> getCompanyData(Map<String, Object> request2Map, User user) {
        Map<String, Object> result = new HashMap<>();
        boolean hasRight = HasRightUtil.hasRight(user, COMPANY_RIGHT, true);
        result.put("hasRight", hasRight);
        if (!hasRight) {
            return result;
        }

        // 根节点
        String root = (String) request2Map.get("root");
        // 显示层级
        String level = (String) request2Map.get("level");
        int currentLevel = Convert.toInt(level, 3);

        // 维度
        String fclass = (String) request2Map.get("fclass");

        // 当前维度
        boolean currentDimension = "0".equals(fclass);


        String fisvitual = (String) request2Map.get("fisvitual");  // 是否显示虚拟组织
        if (StringUtils.isBlank(fisvitual)) {
            fisvitual = "0";
        }


        Set<JyChart> chartList = new LinkedHashSet<>();
        if (currentDimension) {
            if ("0".equals(root)) {
                // 集团
                rs.executeQuery("select * from hrmcompany");
                if (rs.next()) {
                    JyChart item = new JyChart();
                    item.setId("0");
                    item.setFname(rs.getString("companyname"));
                    item.setFtype("0");
                    item.setFnumber(rs.getString("companycode"));
                    item.setFecid(rs.getString("id"));
                    item.setFisvitual("0");
                    item.setExpand("1");
                    chartList.add(item);
                    buildSubCompanyData(item.getFecid(), "0", item.getId(), currentLevel, chartList);
                }
            } else if (root.startsWith("s_")) {
                // 分部
                String chartId = getChartId(root);
                rs.executeQuery("select * from hrmsubcompany where id = ?", chartId);
                if (rs.next()) {
                    JyChart item = new JyChart();
                    item.setId("s_" + rs.getString("id"));
                    item.setFname(rs.getString("subcompanyname"));
                    item.setFtype("1");
                    item.setFnumber(rs.getString("subcompanycode"));
                    item.setFecid(rs.getString("id"));
                    item.setFisvitual("0");
                    item.setExpand("1");
                    chartList.add(item);
                    buildSubCompanyData(rs.getString("companyid"), item.getFecid(), item.getId(), currentLevel, chartList);
                    buildDepartmentData(item.getFecid(), "0", item.getId(), currentLevel, chartList);
                }
            }
        }


        result.put("api_status", true);
        result.put("data", chartList);
        return result;
    }

    @Override
    public Map<String, Object> asyncCompanyData(Map<String, Object> request2Map, User user) {
        String ids = (String) request2Map.get("ids");
        Set<JyChart> chartList = new LinkedHashSet<>();
        if (StringUtils.isNotBlank(ids)) {
            String[] splitIds = ids.split(",");
            for (String splitId : splitIds) {
                if (splitId.startsWith("s_")) {
                    // 分部
                    String chartId = getChartId(splitId);
                    rs.executeQuery("select * from hrmsubcompany where id = ?", chartId);
                    if (rs.next()) {
                        JyChart item = new JyChart();
                        item.setId("s_" + rs.getString("id"));
                        item.setParentId(splitId);
                        item.setFname(rs.getString("subcompanyname"));
                        item.setFtype("1");
                        item.setFnumber(rs.getString("subcompanycode"));
                        item.setFecid(rs.getString("id"));
                        item.setFisvitual("0");
                        item.setExpand("0");
                        chartList.add(item);
                        buildSubCompanyData(rs.getString("companyid"), item.getFecid(), item.getId(), 0, chartList);
                        buildDepartmentData(item.getFecid(), "0", item.getId(), 0, chartList);
                    }
                } else if (splitId.startsWith("d_")) {
                    // 分部
                    String chartId = getChartId(splitId);
                    rs.executeQuery("select * from hrmdepartment where id = ?", chartId);
                    if (rs.next()) {
                        JyChart jyChart = new JyChart();
                        jyChart.setId("d_" + rs.getString("id"));
                        jyChart.setParentId(splitId);
                        jyChart.setFname(rs.getString("departmentmark"));
                        jyChart.setFtype("2");
                        jyChart.setFnumber(rs.getString("departmentcode"));
                        jyChart.setFecid(rs.getString("id"));
                        jyChart.setFisvitual("0");
                        jyChart.setExpand("0");
                        chartList.add(jyChart);
                        String subCompanyId = rs.getString("subcompanyid1");
                        buildDepartmentData(subCompanyId, jyChart.getFecid(), jyChart.getId(), 0, chartList);
                    }
                }
            }
        }

        Map<String, Object> result = new HashMap<>();
        result.put("api_status", true);
        result.put("data", chartList);
        return result;
    }

    @Override
    public Map<String, Object> getUserData(Map<String, Object> request2Map, User user) {
        Map<String, Object> result = new HashMap<>();
        boolean hasRight = HasRightUtil.hasRight(user, USER_RIGHT, true);
        result.put("hasRight", hasRight);
        if (!hasRight) {
            return result;
        }
        String root = (String) request2Map.get("root");  // 根节点
        String level = (String) request2Map.get("level"); // 显示层级
        int currentLevel = Convert.toInt(level, 3);
        // 维度
        String fclass = (String) request2Map.get("fclass");

        // 当前维度
        boolean currentDimension = "0".equals(fclass);

        String fisvitual = (String) request2Map.get("fisvitual");  // 是否显示虚拟组织
        if (StringUtils.isBlank(fisvitual)) {
            fisvitual = "0";
        }


        Set<JyChart> chartList = new LinkedHashSet<>();
        if (currentDimension) {
            if ("0".equals(root)) {
                // 集团
                rs.executeQuery("select * from hrmcompany");
                if (rs.next()) {
                    JyCompanyChart item = new JyCompanyChart();
                    item.setId("0");
                    item.setFname(rs.getString("companyname"));
                    item.setFtype("0");
                    item.setFnumber(rs.getString("companycode"));
                    item.setFecid(rs.getString("id"));
                    item.setFisvitual("0");
                    item.setExpand("1");
                    chartList.add(item);
                    buildSubCompanyUserData(item.getFecid(), "0", item.getId(), currentLevel, chartList);
                }
            } else if (root.startsWith("s_")) {
                // 分部
                String chartId = getChartId(root);
                rs.executeQuery("select * from hrmsubcompany where id = ?", chartId);
                if (rs.next()) {
                    JySubCompanyChart item = new JySubCompanyChart();
                    item.setId("s_" + rs.getString("id"));
                    item.setFname(rs.getString("subcompanyname"));
                    item.setFtype("1");
                    item.setFnumber(rs.getString("subcompanycode"));
                    item.setFecid(rs.getString("id"));
                    item.setFisvitual("0");
                    item.setExpand("1");
                    chartList.add(item);
                    buildSubCompanyUserData(rs.getString("companyid"), item.getFecid(), item.getId(), currentLevel, chartList);
                    buildDepartmentUserData(item.getFecid(), "0", item.getId(), currentLevel, chartList);
                }
            }
        }


        result.put("api_status", true);
        result.put("data", chartList);
        return result;
    }

    @Override
    public Map<String, Object> asyncUserData(Map<String, Object> request2Map, User user) {
        String ids = (String) request2Map.get("ids");
        Set<JyChart> chartList = new LinkedHashSet<>();
        if (StringUtils.isNotBlank(ids)) {
            String[] splitIds = ids.split(",");
            for (String splitId : splitIds) {
                if (splitId.startsWith("s_")) {
                    // 分部
                    String chartId = getChartId(splitId);
                    rs.executeQuery("select * from hrmsubcompany where id = ?", chartId);
                    if (rs.next()) {
                        JySubCompanyChart item = new JySubCompanyChart();
                        item.setId("s_" + rs.getString("id"));
                        item.setParentId(splitId);
                        item.setFname(rs.getString("subcompanyname"));
                        item.setFtype("1");
                        item.setFnumber(rs.getString("subcompanycode"));
                        item.setFecid(rs.getString("id"));
                        item.setFisvitual("0");
                        item.setExpand("0");
                        chartList.add(item);
                        buildSubCompanyUserData(rs.getString("companyid"), item.getFecid(), item.getId(), 0, chartList);
                        buildDepartmentUserData(item.getFecid(), "0", item.getId(), 0, chartList);
                    }
                } else if (splitId.startsWith("d_")) {
                    // 部门
                    String chartId = getChartId(splitId);
                    rs.executeQuery("select * from hrmdepartment where id = ?", chartId);
                    if (rs.next()) {
                        JyDepartmentChart jyChart = new JyDepartmentChart();
                        jyChart.setId("d_" + rs.getString("id"));
                        jyChart.setParentId(splitId);
                        jyChart.setFname(rs.getString("departmentmark"));
                        jyChart.setFtype("2");
                        jyChart.setFnumber(rs.getString("departmentcode"));
                        jyChart.setFecid(rs.getString("id"));
                        jyChart.setFisvitual("0");
                        jyChart.setExpand("0");
                        chartList.add(jyChart);
                        String subCompanyId = rs.getString("subcompanyid1");
                        buildDepartmentUserData(subCompanyId, jyChart.getFecid(), jyChart.getId(), 0, chartList);
                        buildJobUserData(jyChart.getFecid(), jyChart.getId(), 0, chartList);
                    }
                }else if (splitId.startsWith("j_")) {
                    // 岗位
                    String chartId = getChartId(splitId);
                    rs.executeQuery("select * from jcl_org_job where id = ?", chartId);
                    if (rs.next()) {
                        JyJobChart jyChart = new JyJobChart();
                        jyChart.setId("j_" + rs.getString("id"));
                        jyChart.setParentId(splitId);
                        jyChart.setFname(rs.getString("job_name"));
                        jyChart.setFtype("3");
                        jyChart.setFnumber(rs.getString("job_no"));
                        jyChart.setFecid(rs.getString("id"));
                        jyChart.setFisvitual("0");
                        jyChart.setExpand("0");
                        chartList.add(jyChart);
                        buildResourceUserData(jyChart.getFecid(), jyChart.getId(), 0, chartList);
                    }
                }
            }
        }

        Map<String, Object> result = new HashMap<>();
        result.put("api_status", true);
        result.put("data", chartList);
        return result;
    }

    void buildSubCompanyData(String companyId, String parentId, String chartPid, int currentLevel, Set<JyChart> chartList) {
        if (currentLevel < 0) {
            return;
        }
        rs.executeQuery("select * from hrmsubcompany where companyid = " + companyId + " and ifnull(supsubcomid,0) = " + parentId + " and  ifnull(canceled,0) = 0 order by showorder ,showOrderOfTree");
        List<JyChart> subCompanyList = new ArrayList<>();
        while (rs.next()) {
            JyChart item = new JyChart();
            item.setId("s_" + rs.getString("id"));
            item.setParentId(chartPid);
            item.setFname(rs.getString("subcompanyname"));
            item.setFtype("1");
            item.setFnumber(rs.getString("subcompanycode"));
            item.setFecid(rs.getString("id"));
            item.setFisvitual("0");
            item.setExpand(currentLevel > 0 ? "1" : "0");
            subCompanyList.add(item);
        }
        chartList.addAll(subCompanyList);

        for (JyChart jyChart : subCompanyList) {
            buildSubCompanyData(companyId, jyChart.getFecid(), jyChart.getId(), currentLevel - 1, chartList);
            buildDepartmentData(jyChart.getFecid(), "0", jyChart.getId(), currentLevel - 1, chartList);
        }
    }

    void buildDepartmentData(String subCompanyId, String parentId, String chartPid, int currentLevel, Set<JyChart> chartList) {
        if (currentLevel < 0) {
            return;
        }
        rs.executeQuery("select * from hrmdepartment where subcompanyid1 = " + subCompanyId + " and ifnull(supdepid,0) = " + parentId + " and  ifnull(canceled,0) = 0 order by showorder ,showorderoftree");
        List<JyChart> departmentList = new ArrayList<>();
        while (rs.next()) {
            JyChart item = new JyChart();
            item.setId("d_" + rs.getString("id"));
            item.setParentId(chartPid);
            item.setFname(rs.getString("departmentmark"));
            item.setFtype("2");
            item.setFnumber(rs.getString("departmentcode"));
            item.setFecid(rs.getString("id"));
            item.setFisvitual("0");
            item.setExpand(currentLevel > 0 ? "1" : "0");
            departmentList.add(item);
        }
        chartList.addAll(departmentList);
        for (JyChart jyChart : departmentList) {
            buildDepartmentData(subCompanyId, jyChart.getFecid(), jyChart.getId(), currentLevel - 1, chartList);
        }
    }

    void buildSubCompanyUserData(String companyId, String parentId, String chartPid, int currentLevel, Set<JyChart> chartList) {
        if (currentLevel < 0) {
            return;
        }
        rs.executeQuery("select * from hrmsubcompany where companyid = " + companyId + " and ifnull(supsubcomid,0) = " + parentId + " and  ifnull(canceled,0) = 0 order by showorder ,showOrderOfTree");
        List<JySubCompanyChart> subCompanyList = new ArrayList<>();
        while (rs.next()) {
            JySubCompanyChart item = new JySubCompanyChart();
            item.setId("s_" + rs.getString("id"));
            item.setParentId(chartPid);
            item.setFname(rs.getString("subcompanyname"));
            item.setFtype("1");
            item.setFnumber(rs.getString("subcompanycode"));
            item.setFecid(rs.getString("id"));
            item.setFisvitual("0");
            item.setExpand(currentLevel > 0 ? "1" : "0");
            subCompanyList.add(item);
        }
        chartList.addAll(subCompanyList);

        for (JySubCompanyChart jyChart : subCompanyList) {
            buildSubCompanyUserData(companyId, jyChart.getFecid(), jyChart.getId(), currentLevel - 1, chartList);
            buildDepartmentUserData(jyChart.getFecid(), "0", jyChart.getId(), currentLevel - 1, chartList);
        }
    }

    void buildDepartmentUserData(String subCompanyId, String parentId, String chartPid, int currentLevel, Set<JyChart> chartList) {
        if (currentLevel < 0) {
            return;
        }
        String sql = "select * from hrmdepartment where  subcompanyid1 = " + subCompanyId + " and ifnull(supdepid,0) = " + parentId + " and  ifnull(canceled,0) = 0 order by showorder ,showorderoftree";
        rs.executeQuery(sql);
        List<JyDepartmentChart> departmentList = new ArrayList<>();
        while (rs.next()) {
            JyDepartmentChart item = new JyDepartmentChart();
            item.setId("d_" + rs.getString("id"));
            item.setParentId(chartPid);
            item.setFname(rs.getString("departmentmark"));
            item.setFtype("2");
            item.setFnumber(rs.getString("departmentcode"));
            item.setFecid(rs.getString("id"));
            item.setFisvitual("0");
            item.setExpand(currentLevel > 0 ? "1" : "0");
            departmentList.add(item);
        }
        chartList.addAll(departmentList);
        for (JyDepartmentChart jyChart : departmentList) {
            buildDepartmentUserData(subCompanyId, jyChart.getFecid(), jyChart.getId(), currentLevel - 1, chartList);
            buildJobUserData(jyChart.getFecid(), jyChart.getId(), currentLevel - 1, chartList);
        }
    }

    void buildJobUserData(String departmentId, String chartPid, int currentLevel, Set<JyChart> chartList) {
        if (currentLevel < 0) {
            return;
        }
        rs.executeQuery("select * from jcl_org_job where ec_department = ? and ifnull(forbidden_tag,0)=0 order by show_order ", departmentId);
        List<JyJobChart> jobList = new ArrayList<>();
        while (rs.next()) {
            JyJobChart item = new JyJobChart();
            item.setId("j_" + rs.getString("id"));
            item.setParentId(chartPid);
            item.setFname(rs.getString("job_name"));
            item.setFtype("3");
            item.setFnumber(rs.getString("job_no"));
            item.setFecid(rs.getString("id"));
            item.setFisvitual("0");
            item.setExpand(currentLevel > 0 ? "1" : "0");
            jobList.add(item);
        }
        chartList.addAll(jobList);
        for (JyJobChart jyChart : jobList) {
            // 查询人员
            buildResourceUserData(jyChart.getFecid(), jyChart.getId(), currentLevel - 1, chartList);
        }
    }


    void buildResourceUserData(String jobId, String chartPid, int currentLevel, Set<JyChart> chartList) {
        if (currentLevel < 0) {
            return;
        }
        rs.executeQuery("select a.id,a.lastname from hrmresource a inner join cus_fielddata b  on a.id = b.id and b.scope ='hrmcustomfieldbyinfotype' and b.scopeid ='-1' where a.status < 4 and  b.field100002 = ?", jobId);
        List<JyResourceChart> resourceList = new ArrayList<>();
        while (rs.next()) {
            JyResourceChart item = new JyResourceChart();
            item.setId("r_" + rs.getString("id"));
            item.setParentId(chartPid);
            item.setFname(rs.getString("lastname"));
            item.setFtype("4");
            item.setFnumber(rs.getString("hrmresource"));
            item.setFecid(rs.getString("id"));
            item.setFisvitual("0");
            item.setExpand(currentLevel > 0 ? "1" : "0");
            resourceList.add(item);
        }
        chartList.addAll(resourceList);
    }


    public static String getChartId(String id) {
        if (StringUtils.isNotBlank(id)) {
            String[] split = id.split("_");
            if (split.length == 2) {
                return split[1];
            }
        }
        return "";
    }

}
